#!/usr/bin/perl -w
#
# btl-pirata-fr24.pl
#
#  Genere un fichier btl a partir des fichiers
#  Seabird .btl
#
# $Id J Grelet avril 2014$
#
use strict; # necessite de declarer toutes les variables globales
#use diagnostics;

# use open qw(:std :utf8); 
# binmode(STDERR, ":utf8");
use Time::Local;
use Date::Manip;
use Getopt::Long;
use File::Basename;
use Data::Dumper;
use Oceano::Seawater;
use Oceano::Convert;
use Oceano::Seabird;

#------------------------------------------------------------------------------
# Les variables globales
#------------------------------------------------------------------------------
our $VERSION       = '1.0';
our $AUTHOR        = 'jgrelet IRD juillet 2012 PIRATA-FR24 cruise';
my  $DEBUG         = 0;
my  $ECHO          = 0;
my  $dtd           = 0;

# surcharge par la valeurs saisies sur la ligne de commande
my  $version;
my  $help;

my  $btl_file     = 'pirata-fr24.btl';
my  $xml_btl_file = 'pirata-fr24_btl.xml';
my  $odv_file     = 'pirata-fr24_btl_odv.txt';

my  $institut      = 'IRD';
my  $type          = 'SBE32';
my  $sn            = 'SBE32-12-IRD';
my  $pi            = 'BOURLES';

# en dur pour pirata-fr24 :
my  $cycle_mesure  = 'PIRATA-FR24';
my  $plateforme    = 'LE SUROIT';
my  $CONTEXTE      = "AMMA";
my  $TIMEZONE      = "GMT";
my  $FORMAT_DATE   = "DMY";
my  $CODE          = "1A";
my  $date_debut    = "09/04/2014";
my  $date_fin      = "21/05/2014";
my  $cruisePrefix  = "fr24";
my  $title_summary = "Water sample during PIRATA-FR24 cruise with 11 levels";
my  $comment       = "CTD bottles water sampling with temperature, salinity and oxygen from primary and secondary sensors, PSAL and DOX2 are empty and reserved for chimical analyses";


my $code     = -1;  # code pour l'entete


# fichiers bouteille seabird .btl
my  $sbe_file;
my ($s_date, $e_date, $e_time, $iso_date);
my ($station, $num_btl);
my ($lat, $lat_pos, $lat_deg, $lat_min, $lat_hemi);
my ($long, $long_pos, $long_deg, $long_min, $long_hemi);
my ($julien, $date, $mois, $jour, $annee, $heure, $min, $time,$ETDD);
my ($bottle,$PSA1,$PSA2,$DO11,$DO12,$DO21,$DO22,$DVT1,$DVT2,$PRES,$DEPH,
    $TE01,$TE02,$DOV1,$DOV2,$CND1,$CND2,$TUR3,$FLU2,$TUR4,$LGH4,$LGHT,$LGH5);
my ($Potemp090C, $Potemp190C); # temperature potentielle

my  $type_odv      = 'B';  # station-type (either B, C or X for bottle, CTD or XBT data
my  $bottom_depth  = 1e36;

#------------------------------------------------------------------------------
# version()
#------------------------------------------------------------------------------	
sub version() {
  print "Version: $VERSION\nAuthor: $AUTHOR\n\n";	
  exit 1;
}	

#------------------------------------------------------------------------------
# usage()
#------------------------------------------------------------------------------	
sub usage() {
  print STDERR "\nusage: btl-pirata-fr24.pl [options] <files>\n\n";
  print STDERR   "Options:\n    --help                 Display this help message\n";
  print STDERR   "    --version              program version\n";
  print STDERR   "    --debug                debug info\n";
  print STDERR   "    --echo                 display filenames processed\n";
  print STDERR   "examples:\n\$ perl btl-pirata-fr24.pl --echo data/btl/fr24*.btl\n"; 
  exit 1;
}

#------------------------------------------------------------------------------
# get_options()
# analyse les options
#------------------------------------------------------------------------------	
sub get_options() {
  
  &GetOptions ("echo"            => \$ECHO,  	       
               "local"           => \$dtd,  
               "debug"           => \$DEBUG,  
               "version"         => \$version,  
               "help"            => \$help)  or &usage;  
       
  &version if $version;	
  &usage   if $help;
}

#------------------------------------------------------------------------------
# write XML header to output _ctd.xml file using oceano.dtd 
#------------------------------------------------------------------------------
sub write_xml_header { 
#  my $today = &dateFormat(undef,"%d/%m/%Y");
  
  print  XML_BTL_FILE '<?xml version="1.0" encoding="ISO-8859-1"?>' . "\n"; 
  # les commentaires ne sont pas acceptés par XML Toolbox Matlab de Geodise
  if ( defined $dtd ) {
    #print  XML_BTL_FILE '<!DOCTYPE OCEANO SYSTEM "M:\ACQUISIT\oceano.dtd">' . "\n";
    print  XML_BTL_FILE '<!DOCTYPE OCEANO SYSTEM "/m/PIRATA-FR24/local/oceano.dtd">' . "\n";
    #print  XML_BTL_FILE '<!DOCTYPE OCEANO PUBLIC "-//US191//DTD OCEANO//FR" "http://antea/acquisit/oceano.dtd">' . "\n";
  } else {  
    print  XML_BTL_FILE '<!DOCTYPE OCEANO PUBLIC "-//US191//DTD OCEANO//FR" "http://www.brest.ird.fr/us191/database/oceano.dtd">' . "\n";
  }
  print  XML_BTL_FILE '<OCEANO TYPE="PROFIL">' . "\n";
  print  XML_BTL_FILE "  <ENTETE>\n";
  print  XML_BTL_FILE "    <PLATEFORME>\n";
  print  XML_BTL_FILE "      <LIBELLE>$plateforme</LIBELLE>\n";  
  print  XML_BTL_FILE "    </PLATEFORME>\n";
  print  XML_BTL_FILE "    <CYCLE_MESURE CONTEXTE=\"$CONTEXTE\" TIMEZONE=\"$TIMEZONE\" FORMAT=\"$FORMAT_DATE\">\n";  
  print  XML_BTL_FILE "      <LIBELLE>$cycle_mesure</LIBELLE>\n";  
  print  XML_BTL_FILE "      <DATE_DEBUT>$date_debut</DATE_DEBUT>\n";  
  print  XML_BTL_FILE "      <DATE_FIN>$date_fin</DATE_FIN>\n";  
  print  XML_BTL_FILE "      <INSTITUT>$institut</INSTITUT>\n";  
  print  XML_BTL_FILE "      <RESPONSABLE>$pi</RESPONSABLE>\n"; 
#  print  XML_BTL_FILE "      <ACQUISITION LOGICIEL=\"SEASAVE\" VERSION=\"$seasave_version\"></ACQUISITION>\n"; 
#  print  XML_BTL_FILE "      <TRAITEMENT LOGICIEL=\"SBEDATAPROCESSING\" VERSION=\"$seasave_version\"></TRAITEMENT>\n"; 
#  print  XML_BTL_FILE "      <VALIDATION LOGICIEL=\"$0\" VERSION=\"$VERSION\" DATE=\"$today\" OPERATEUR=\"$pi\" CODIFICATION=\"OOPC\">\n";
#  print  XML_BTL_FILE "        <CODE>$CODE</CODE>\n";	    
  print  XML_BTL_FILE "        <COMMENTAIRE>$comment</COMMENTAIRE>\n";
  print  XML_BTL_FILE "        <TITLE_SUMMARY>$title_summary</TITLE_SUMMARY>\n";
  print  XML_BTL_FILE "    </CYCLE_MESURE>\n";  
  print  XML_BTL_FILE "    <INSTRUMENT TYPE=\"$type\" NUMERO_SERIE=\"$sn\">\n"; 
  print  XML_BTL_FILE "    </INSTRUMENT>\n";  
  print  XML_BTL_FILE "  </ENTETE>\n";  
  print  XML_BTL_FILE "  <DATA>\n";  
}

#------------------------------------------------------------------------------
# fonctions de calcul de la position/date
#------------------------------------------------------------------------------
sub position {
  my($deg,$min,$hemi) = @_;
  my $sign = 1;
  if( $hemi eq "S" || $hemi eq "W") {
    $sign = -1;
  }
  my $tmp = $min;
  $min = abs $tmp;
  my $sec = ($tmp - $min ) * 100;
  return( ( $deg + ( $min + $sec / 100 ) / 60 ) * $sign ); 
}

#------------------------------------------------------------------------------
#------------------------------------------------------------------------------
#sub julian {
#  my($jj,$h,$m) = @_;
#  my $tmp = (($h * 60) + $m ) / 1440;
#  return( $jj + $tmp ); 
#}

#------------------------------------------------------------------------------
# entete ODV
#------------------------------------------------------------------------------
sub entete_odv { 
  my $today = &dateFormat(undef,"%Y-%m-%dT%H:%M:%S");
  print  ODV_FILE "//ODV Spreadsheet file : $odv_file\n"; 
  print  ODV_FILE "//Data treated : $today\n"; 
  print  ODV_FILE "//<DataType>Profiles</DataType>\n";
  print  ODV_FILE "//<InstrumentType>SBE32 standard 12 Niskin bottles</InstrumentType>\n";
  print  ODV_FILE "//<Source>M:\\PIRATA-FR24\\data-processing\\CTD</Sources>\n"; 
  print  ODV_FILE "//<Creator>Jacques.Grelet\@ird.fr</Creator>\n";
  print  ODV_FILE "//\n"; 
  print  ODV_FILE "Cruise\tStation\tType\tyyyy-mm-ddThh:mm:ss\tLongitude [degrees_east]\tLatitude [degrees_north]\tBot. Depth [m]\tBTL\tPres [db]\tTE01 [C]\tTE02 [C]\tPSA1 [Psu]\tPSA2 [Psu]\tDO12 [micromole/kg]\tDO22 [micromole/kg]\tFLU2 [milligram/m3]\t TUR3 [%]\tPotemp090[C]\tCPotemp190C[C]\n";
}


#------------------------------------------------------------------------------
# Debut du programme principal
#------------------------------------------------------------------------------
&Date_Init( "TZ=UTC" );
&usage if( $#ARGV == -1);
&get_options;

# ouvre les fichiers de sortie
open( BTL_FILE, "+> $btl_file" ) or die "Can't open file : $btl_file\n";
open( XML_BTL_FILE, "+> $xml_btl_file" ) or die "Can't open file : $xml_btl_file\n";
open( ODV_FILE,     "+> $odv_file" )   or die "Can't open file : $odv_file\n";

print BTL_FILE " St  Date_deb  Heure_deb  Date_fin Heure_fin  Latitude  Longitude  Nb_btl\n\n"; 
&entete_odv;
&write_xml_header;
print XML_BTL_FILE "PRFL  BOTL  PRES   DEPH     ETDD     TE01    TE02    PSAL   PSA1    PSA2   CND1     CND2    DO11    DO21    DOX2    DO12     DO22     DOV1    DOV2      DVT1    DVT2    TUR3   FLU2    Potemp090C Potemp190C\n";

# parcourt des fichiers .btl passes en argument sur la ligne de commande
for( my $i = 0; $i <= $#ARGV; $i++ ){
  my $fileName = $ARGV[$i];

  # ouverture du fichier
  open( DATA_FILE, $fileName );
  print STDERR  "\nLit: $fileName  = " if $DEBUG;

  # recupere le numero de la station dans le nom du fichier
  ($station) = ($fileName =~ m/$cruisePrefix(\d{3})/i);

  $num_btl = 0;

  # on lit le fichier bouteille
  while( <DATA_FILE> ){  
    # decode la date et heure de debut de station	  
    if( /System UpLoad Time =\s+(\.*)/)	{ # a modifier suivant le contexte
      ($time)  = /System UpLoad Time =\s+(\w+\s+\d+\s+\d+\s+\d+:\d+:\d+)/;	
    $date=&ParseDate($time);
    # transforme le day_of_year en julian day
    $julien = &UnixDate($time,"%j") -1;
    $julien = &julian( $julien, &UnixDate($date,"%H"),&UnixDate($date,"%M"),&UnixDate($date,"%S") );
    $s_date = &UnixDate($date,"%d/%m/%Y %H:%M:%S");
    }
    if( /NMEA Latitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ )	{
      ($lat_deg, $lat_min, $lat_hemi) = split " ", $1;
      $lat_pos = &position($lat_deg, $lat_min, $lat_hemi);
    }
    if( /NMEA Longitude\s*=\s*(\d+\s+\d+.\d+\s+\w)/ ) {
      ($long_deg, $long_min, $long_hemi) = split " ", $1;
      $long_pos = &position($long_deg, $long_min, $long_hemi);
      printf XML_BTL_FILE "%05d  %d %7.3f %7.4f %8.4f %s 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36 1e36\n",$station,$code,$julien,$lat_pos, $long_pos,&UnixDate($date,"%q");
    }
    # decodage du fond (sonde)
    if( /Bottom Depth\s*:\s*(\d*)\s*\S*/i ) {
      $bottom_depth = ($1 eq '') ?  1e+36 : $1; 
    }  
     # decode la date de chaque prelevement et extrait les donnees dans la ligne correspondante
     # les parametres suivants doivent etre presents:
     # Bottle Date Sal00 Sal11 Sbeox0ML/L Sbeox0Mm/Kg Sbeox1ML/L Sbeox1Mm/Kg Sbeox0dOV/dT Sbeox1dOV/dT PrDM DepSM 
     # T090C T190C C0S/m C1S/m Sbeox0V Sbeox1V (optionnels) FlECO-AFL Xmiss
     # amop
     # Bottle Date Sal00 Sal11 Sbeox0ML/L Sbeox0Mm/Kg Sbeox1ML/L Sbeox1Mm/Kg TimeJ PrDM DepSM T090C T190C
     # C0S/m C1S/m Sbeox0V Sbeox1V Upoly0 FlC CStarAt0 CStarTr0 Spar Par Cpar Sbeox0dOV/dT Sbeox1dOV/dT
     # Position Time    
     # TUR3 -> CStarTr0  (transmissiometre Wetlab %)
     # TUR4 -> Upoly0 (turbidimètre)
     # LGHT;LIGHT IRRADIANCE IMMERGED PAR;micromole photon/(m2.s);0;4000;%8.3lf;9999.999;
     # LGH4;LIGHT IRRADIANCE SURFACE PAR;micromole photon/(m2.s);0;3000;%8.3lf;9999.999;
     # LGH5 -> CPar (%)
     # FLU2 -> FlC (Chelsea Acquatracka III) 
     if( /^\s+\d+\s+\w+\s+\d+\s+\d+/) {
       #($time) = /\s+\d+\s+(\w+\s+\d+\s+\d+)/;	
      ($bottle,$mois,$jour,$annee,$PSA1,$PSA2,$DO11,$DO21,$DO12,$DO22,
	 $Potemp090C,$Potemp190C,$ETDD, $PRES,$DEPH,$TE01,$TE02,
         $CND1,$CND2,$DOV1,$DOV2,$DVT1,$DVT2,$FLU2,$TUR3) = split;
       $date = &ParseDate($mois.' '.$jour.' '.$annee);
       $e_date = &UnixDate($date,"%d/%m/%Y");	
       $iso_date = &UnixDate($date,"%Y-%m-%dT");	
       # a changer eventuellement si LADCP et choix "table driven" dans seaseave
       $num_btl += 1;
       printf XML_BTL_FILE "%05d  %2d %6.1f %6.1f %10.6f %7.4f %7.4f   1e36 %7.4f %7.4f %7.5f %7.5f %7.4f %7.4f   1e36 %8.3f %8.3f %8.5f %8.5f %+7.5f %+7.5f %6.2f %+6.4f   %7.4f   %7.4f\n",
       $station,$num_btl,$PRES,$DEPH,$ETDD,$TE01,$TE02,$PSA1,$PSA2,$CND1,$CND2,$DO11,$DO12,$DO21,$DO22,$DOV1,$DOV2,$DVT1,$DVT2,$TUR3,$FLU2,$Potemp090C,$Potemp190C;
     }
     # decode l'heure sur la ligne suivante
     if( /\s+(\d+:\d+:\d+)/) {
       ($time) = /\s+(\d+:\d+:\d+)/;	
       $date = &ParseDate($time);
       $e_time = &UnixDate($date,"%H:%M:%S");	
       if( $num_btl >= 1) {
         printf ODV_FILE "%s\t%05d\t%s\t%s\t%8.4f\t%7.4f",
         $cycle_mesure, $station, $type_odv, $iso_date.&UnixDate($date,"%H:%M:%S"),
         $long_pos, $lat_pos;
         printf ODV_FILE ($bottom_depth > 1e35) ? "\t" : "\t%6.1f", $bottom_depth;
         printf ODV_FILE "\t%2d",     $num_btl;         
         printf ODV_FILE "\t%6.1f",   $PRES;         
         printf ODV_FILE ($TE01 > 1e35) ? "\t" : "\t%7.4f", $TE01;
         printf ODV_FILE ($TE02 > 1e35) ? "\t" : "\t%7.4f", $TE02;
         printf ODV_FILE ($PSA1 > 1e35) ? "\t" : "\t%7.4f", $PSA1;
         printf ODV_FILE ($PSA2 > 1e35) ? "\t" : "\t%7.4f", $PSA2;
         printf ODV_FILE "\t%6.3f",   $DO21;
         printf ODV_FILE "\t%7.2f",   $DO22;
         printf ODV_FILE "\t%6.3f",   $FLU2;
         printf ODV_FILE "\t%6.4f",   $TUR3;
         printf ODV_FILE "\t%7.4f",   $Potemp090C;	 
         printf ODV_FILE "\t%7.4f\n", $Potemp190C;	 
       }  
     }
  }
  # pour chaque fichier, on garde la date et heure de la derniere bouteille
  # impression a l'ecran
  printf STDERR "%4d  %s  %s %s  %02d%c%05.2f %s  %03d\xB0%05.2f %s  %2d", $station, 
    $s_date,$e_date, $e_time, $lat_deg, 176, $lat_min, $lat_hemi, $long_deg, 
    $long_min, $long_hemi,  $num_btl  if $ECHO;
  printf STDERR "\n" if not $DEBUG and $ECHO; 

  # impression dans le fichier 
  printf BTL_FILE "%5d %s %s %s %02d%c%05.2f %s %03d%c%05.2f %s  %2d\n", 
    $station, $s_date, $e_date, $e_time, $lat_deg, 176, $lat_min, $lat_hemi,
    $long_deg, 176, $long_min, $long_hemi, $num_btl; 
}  

printf STDERR "\n";

print  XML_BTL_FILE "  </DATA>\n";  
print  XML_BTL_FILE "</OCEANO>\n";  

close BTL_FILE;
close XML_BTL_FILE;
close ODV_FILE;
